@@ -16,26 +16,25 @@
 # As of 12/04/2022 Multi_mat scripts only suport "slow" mode of UB matrix computation
 # =============================================================================
 
+## PLease install pandas and tables library from PYPI to launch this script
 # In[1]:
 if __name__ == "__main__":    
 
     ## Import modules used for this Notebook
     import numpy as np
     import os
-    import multiprocessing
-    from multiprocessing import cpu_count
+    from multiprocessing import Process, Queue, cpu_count
     import time, datetime
     import glob, re
     import configparser
     from itertools import accumulate
     ## if LaueToolsNN is properly installed
-    from lauetoolsnn.utils_lauenn import get_multimaterial_detail, new_MP_multimat_function, resource_path, global_plots_MM
+    from lauetoolsnn.utils_lauenn import get_multimaterial_detail, new_MP_multimat_function_v1, resource_path, global_plots_MM
     from lauetoolsnn.lauetools import dict_LaueTools as dictLT
     from lauetoolsnn.NNmodels import read_hdf5
-    
     import _pickle as cPickle
-    from tqdm import tqdm
     import pandas as pd
+    from tqdm import tqdm
     
     ncpu = cpu_count()
     print("Number of CPUs available : ", ncpu)
@@ -227,7 +226,88 @@ if __name__ == "__main__":
     ct = time.time()
     now = datetime.datetime.fromtimestamp(ct)
     c_time = now.strftime("%Y-%m-%d_%H-%M-%S")   
+    #%% Some useful functions
+    def chunker_list(seq, size):
+        return (seq[i::size] for i in range(size))
     
+    def reshape(a):
+        m,n,r = a.shape
+        return a.reshape((m,n*r))
+    
+    def write_hdf5(ubmat, rotation_matrix, strain_matrix, strain_matrixs, match_rate,
+                   filenames, spots_len, iR_pix, fR_pix, colx, coly, col, mat_global,
+                   results_hdf5_filepath, initialize=True):
+        for i in range(ubmat):
+            ## Rotation matrix components
+            a = reshape(rotation_matrix[i][0])
+            columnsa=['R11', 'R12', 'R13', 'R21', 'R22', 'R23', 'R31', 'R32', 'R33']
+            ## Strain crystal frame components
+            b = reshape(strain_matrix[i][0])
+            columnsb=['Ceps11', 'Ceps12', 'Ceps13', 'Ceps21', 'Ceps22', 'Ceps23', 'Ceps31', 'Ceps32', 'Ceps33']
+            ## Strain sample frame components
+            c = reshape(strain_matrixs[i][0])
+            columnsc=['Seps11', 'Seps12', 'Seps13', 'Seps21', 'Seps22', 'Seps23', 'Seps31', 'Seps32', 'Seps33']
+            ## matching rate
+            d = match_rate[i][0]
+            columnsd=['MatchinRate']
+            ## filename
+            e = np.chararray(d.shape, itemsize=1000)
+            for jj, ii in enumerate(filenames):
+                e[jj] = ii
+            columnse=['filename']
+            ## spots_len
+            f = spots_len[i][0]
+            columnsf=['NbofSpots']
+            ## initial residues
+            g = iR_pix[i][0]
+            columnsg=['initialResidue']
+            ## final residues
+            h = fR_pix[i][0]
+            columnsh=['finalResidue']
+            ## colx
+            f1 = colx[i][0]
+            columnsf1=['ipfXr', 'ipfXg', 'ipfXb']
+            ## coly
+            g1 = coly[i][0]
+            columnsg1=['ipfYr', 'ipfYg', 'ipfYb']
+            ## colz
+            h1 = col[i][0]
+            columnsh1=['ipfZr', 'ipfZg', 'ipfZb']
+            ## convert matglobal to string of material label
+            mat_id = mat_global[i][0]
+            mat_label = np.chararray(mat_id.shape, itemsize=1000)
+            for jj, ii in enumerate(mat_id):
+                mat_label[jj] = material_[int(ii-1)]
+            columns11=['material_label']
+            columns = columnsa+columnsb+columnsc+columnsd+columnse+columnsf+\
+                        columnsg+columnsh+columnsf1+columnsg1+columnsh1+columns11
+            out_array = np.hstack((a,b,c,d,e,f,g,h,f1,g1,h1,mat_label))
+            columnstype={'R11':'float64', 'R12':'float64', 'R13':'float64', 'R21':'float64', 
+                           'R22':'float64', 'R23':'float64', 'R31':'float64', 'R32':'float64', 
+                           'R33':'float64', 'Ceps11':'float64', 'Ceps12':'float64', 'Ceps13':'float64', 'Ceps21':'float64', 
+                           'Ceps22':'float64', 'Ceps23':'float64', 'Ceps31':'float64', 'Ceps32':'float64', 
+                           'Ceps33':'float64', 'Seps11':'float64', 'Seps12':'float64', 
+                           'Seps13':'float64', 'Seps21':'float64', 'Seps22':'float64', 'Seps23':'float64', 
+                           'Seps31':'float64', 'Seps32':'float64', 'Seps33':'float64', 'MatchinRate':'float64',
+                           'filename':'object', 'NbofSpots':'float64', 'initialResidue':'float64','finalResidue':'float64', 
+                           'ipfXr':'float64', 'ipfXg':'float64', 'ipfXb':'float64',
+                           'ipfYr':'float64', 'ipfYg':'float64', 'ipfYb':'float64', 'ipfZr':'float64', 
+                           'ipfZg':'float64', 'ipfZb':'float64', 'material_label':'object'}
+            for ij in range(len(columns)):
+                temp_ = out_array[:,ij]
+                temp_ = temp_.reshape((lim_x, lim_y))
+                out_df = pd.DataFrame(temp_)
+                dtype_ = columnstype[columns[ij]]
+                out_df = out_df.astype(dtype=dtype_)
+                try:
+                    out_df.to_hdf(results_hdf5_filepath, key='grain'+str(i)+"/"+columns[ij])
+                except:
+                    continue
+                # if initialize:
+                #     out_df.to_hdf(results_hdf5_filepath, key='grain'+str(i)+"/"+columns[ij])
+                # else:
+                #     out_df.to_hdf(results_hdf5_filepath, key='grain'+str(i)+"/"+columns[ij], append=True, mode='r+')
+                
     #%% ## Step 3: Initialize variables and prepare arguments for multiprocessing module
     
     col = [[] for i in range(int(ubmat))]
@@ -367,8 +447,6 @@ if __name__ == "__main__":
                 crystal,
                 strain_free_parameters,
                 model_annote] for ii in range(count_global)]
-    start_time = time.time()
-    #% Launch multiprocessing prediction     
     
     #% Save results
     save_directory_ = model_direc+"//results_"+prefix_mat+"_"+c_time
@@ -376,125 +454,81 @@ if __name__ == "__main__":
         os.makedirs(save_directory_)
         
     results_hdf5_filepath = save_directory_ + "//results.h5"
-
-    args = zip(valu12)
-    with multiprocessing.Pool(ncpu) as pool:
-        results = pool.starmap(new_MP_multimat_function, tqdm(args, total=len(valu12)))
+    
+    ncpu = cpu_count()
+    _inputs_queue = Queue()
+    _outputs_queue = Queue()
+    _worker_processes = {}
+    for i in range(ncpu):
+        _worker_processes[i]= Process(target=new_MP_multimat_function_v1, args=(_inputs_queue, _outputs_queue, i+1))
+    for i in range(ncpu):
+        _worker_processes[i].start()
         
-        for r in results:
-            r_message_mpdata = r
-            strain_matrix_mpdata, strain_matrixs_mpdata, rotation_matrix_mpdata, col_mpdata,\
-            colx_mpdata, coly_mpdata, match_rate_mpdata, mat_global_mpdata,\
-                cnt_mpdata, meta_mpdata, files_treated_mpdata, spots_len_mpdata, \
-                    iR_pixel_mpdata, fR_pixel_mpdata, best_match_mpdata, check_mpdata = r_message_mpdata
     
-            for i_mpdata in files_treated_mpdata:
-                files_treated.append(i_mpdata)
+
+    chunks = chunker_list(valu12, ncpu)
+    chunks_mp = list(chunks)
+
+    meta = {'t1':time.time()}
+    for ijk in range(int(ncpu)):
+        _inputs_queue.put((chunks_mp[ijk], ncpu, meta))
+
+    start_time = time.time()
     
-            for intmat_mpdata in range(int(ubmat)):
-                check[cnt_mpdata,intmat_mpdata] = check_mpdata[cnt_mpdata,intmat_mpdata]
-                mat_global[intmat_mpdata][0][cnt_mpdata] = mat_global_mpdata[intmat_mpdata][0][cnt_mpdata]
-                strain_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-                strain_matrixs[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrixs_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-                rotation_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = rotation_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
-                col[intmat_mpdata][0][cnt_mpdata,:] = col_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-                colx[intmat_mpdata][0][cnt_mpdata,:] = colx_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-                coly[intmat_mpdata][0][cnt_mpdata,:] = coly_mpdata[intmat_mpdata][0][cnt_mpdata,:]
-                match_rate[intmat_mpdata][0][cnt_mpdata] = match_rate_mpdata[intmat_mpdata][0][cnt_mpdata]
-                spots_len[intmat_mpdata][0][cnt_mpdata] = spots_len_mpdata[intmat_mpdata][0][cnt_mpdata]
-                iR_pix[intmat_mpdata][0][cnt_mpdata] = iR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
-                fR_pix[intmat_mpdata][0][cnt_mpdata] = fR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
-                best_match[intmat_mpdata][0][cnt_mpdata] = best_match_mpdata[intmat_mpdata][0][cnt_mpdata]
-            
-            
-            def reshape(a):
-                m,n,r = a.shape
-                return a.reshape((m,n*r))
+    write_hdf5(ubmat, rotation_matrix, strain_matrix, strain_matrixs, match_rate,
+                   [], spots_len, iR_pix, fR_pix, colx, coly, col, mat_global,
+                   results_hdf5_filepath, initialize=True)
+    
+    ### Update data from multiprocessing
+    pbar = tqdm(total=count_global)
+    
+    unique_count = []
+    while True:
+        
+        if len(np.unique(unique_count)) == count_global:
+            print("All files have been treated")
+            break
+        
+        time.sleep(1)
+        pbar.update(len(np.unique(unique_count)))
+        
+        if not _outputs_queue.empty():            
+            n_range = _outputs_queue.qsize()
+            for _ in range(n_range):
+                r_message_mpdata = _outputs_queue.get()
+                strain_matrix_mpdata, strain_matrixs_mpdata, rotation_matrix_mpdata, col_mpdata,\
+                colx_mpdata, coly_mpdata, match_rate_mpdata, mat_global_mpdata,\
+                    cnt_mpdata, meta_mpdata, files_treated_mpdata, spots_len_mpdata, \
+                        iR_pixel_mpdata, fR_pixel_mpdata, best_match_mpdata, check_mpdata = r_message_mpdata
+
+                for i_mpdata in files_treated_mpdata:
+                    files_treated.append(i_mpdata)
+
+                for intmat_mpdata in range(int(ubmat)):
+                    check[cnt_mpdata,intmat_mpdata] = check_mpdata[cnt_mpdata,intmat_mpdata]
+                    mat_global[intmat_mpdata][0][cnt_mpdata] = mat_global_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    strain_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    strain_matrixs[intmat_mpdata][0][cnt_mpdata,:,:] = strain_matrixs_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    rotation_matrix[intmat_mpdata][0][cnt_mpdata,:,:] = rotation_matrix_mpdata[intmat_mpdata][0][cnt_mpdata,:,:]
+                    col[intmat_mpdata][0][cnt_mpdata,:] = col_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    colx[intmat_mpdata][0][cnt_mpdata,:] = colx_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    coly[intmat_mpdata][0][cnt_mpdata,:] = coly_mpdata[intmat_mpdata][0][cnt_mpdata,:]
+                    match_rate[intmat_mpdata][0][cnt_mpdata] = match_rate_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    spots_len[intmat_mpdata][0][cnt_mpdata] = spots_len_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    iR_pix[intmat_mpdata][0][cnt_mpdata] = iR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    fR_pix[intmat_mpdata][0][cnt_mpdata] = fR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
+                    best_match[intmat_mpdata][0][cnt_mpdata] = best_match_mpdata[intmat_mpdata][0][cnt_mpdata]
             
+                if cnt_mpdata not in unique_count:
+                    pbar.update(1)
+                unique_count.append(cnt_mpdata)
+                
             filenames = list(np.unique(files_treated))
             filenames.sort(key=lambda var:[int(x) if x.isdigit() else x for x in re.findall(r'[^0-9]|[0-9]+', var)])
-
-            dataframe_grains = []
-            for i in range(ubmat):
-                ## Rotation matrix components
-                a = reshape(rotation_matrix[i][0])
-                columnsa=['R11', 'R12', 'R13', 'R21', 'R22', 'R23', 'R31', 'R32', 'R33']
-                
-                ## Strain crystal frame components
-                b = reshape(strain_matrix[i][0])
-                columnsb=['Ceps11', 'Ceps12', 'Ceps13', 'Ceps21', 'Ceps22', 'Ceps23', 'Ceps31', 'Ceps32', 'Ceps33']
-                
-                ## Strain sample frame components
-                c = reshape(strain_matrixs[i][0])
-                columnsc=['Seps11', 'Seps12', 'Seps13', 'Seps21', 'Seps22', 'Seps23', 'Seps31', 'Seps32', 'Seps33']
-                
-                ## matching rate
-                d = match_rate[i][0]
-                columnsd=['MatchinRate']
-                
-                ## filename
-                e = np.chararray(d.shape, itemsize=1000)
-                for jj, ii in enumerate(filenames):
-                    e[jj] = ii
-                columnse=['filename']
-                
-                ## spots_len
-                f = spots_len[i][0]
-                columnsf=['NbofSpots']
-                
-                ## initial residues
-                g = iR_pix[i][0]
-                columnsg=['initialResidue']
-                
-                ## final residues
-                h = fR_pix[i][0]
-                columnsh=['finalResidue']
-                
-                ## colx
-                f1 = colx[i][0]
-                columnsf1=['ipfXr', 'ipfXg', 'ipfXb']
-                
-                ## coly
-                g1 = coly[i][0]
-                columnsg1=['ipfYr', 'ipfYg', 'ipfYb']
-                
-                ## colz
-                h1 = col[i][0]
-                columnsh1=['ipfZr', 'ipfZg', 'ipfZb']
-                
-                ## convert matglobal to string of material label
-                mat_id = mat_global[i][0]
-                mat_label = np.chararray(mat_id.shape, itemsize=1000)
-                for jj, ii in enumerate(mat_id):
-                    mat_label[jj] = material_[int(ii-1)]
-                columns11=['material_label']
-                
-                
-                columns = columnsa+columnsb+columnsc+columnsd+columnse+columnsf+\
-                            columnsg+columnsh+columnsf1+columnsg1+columnsh1+columns11
-                
-                out_array = np.hstack((a,b,c,d,e,f,g,h,f1,g1,h1,mat_label))
-                
-                columnstype={'R11':'float64', 'R12':'float64', 'R13':'float64', 'R21':'float64', 
-                               'R22':'float64', 'R23':'float64', 'R31':'float64', 'R32':'float64', 
-                               'R33':'float64', 'Ceps11':'float64', 'Ceps12':'float64', 'Ceps13':'float64', 'Ceps21':'float64', 
-                               'Ceps22':'float64', 'Ceps23':'float64', 'Ceps31':'float64', 'Ceps32':'float64', 
-                               'Ceps33':'float64', 'Seps11':'float64', 'Seps12':'float64', 
-                               'Seps13':'float64', 'Seps21':'float64', 'Seps22':'float64', 'Seps23':'float64', 
-                               'Seps31':'float64', 'Seps32':'float64', 'Seps33':'float64', 'MatchinRate':'float64',
-                               'filename':'object', 'NbofSpots':'float64', 'initialResidue':'float64','finalResidue':'float64', 
-                               'ipfXr':'float64', 'ipfXg':'float64', 'ipfXb':'float64',
-                               'ipfYr':'float64', 'ipfYg':'float64', 'ipfYb':'float64', 'ipfZr':'float64', 
-                               'ipfZg':'float64', 'ipfZb':'float64', 'material_label':'object'}
-                
-                for ij in range(len(columns)):
-                    temp_ = out_array[:,ij]
-                    temp_ = temp_.reshape((lim_x, lim_y))
-                    out_df = pd.DataFrame(temp_)
-                    dtype_ = columnstype[columns[ij]]
-                    out_df = out_df.astype(dtype=dtype_)
-                    out_df.to_hdf(results_hdf5_filepath, key='grain'+str(i)+"/"+columns[ij])
             
+            write_hdf5(ubmat, rotation_matrix, strain_matrix, strain_matrixs, match_rate,
+                           filenames, spots_len, iR_pix, fR_pix, colx, coly, col, mat_global,
+                           results_hdf5_filepath, initialize=False)
             
     ## intermediate saving of pickle objects with results
     np.savez_compressed(save_directory_+ "//results.npz", 
@@ -517,4 +551,6 @@ if __name__ == "__main__":
                      iR_pix, fR_pix, save_directory_, material_,
                      match_rate_threshold=5, bins=30)
     except:
-        print("Error in the global plots module")
\ No newline at end of file
+        print("Error in the global plots module")
+        
+    print("Took ", time.time()-start_time,"s")
\ No newline at end of file
