@@ -30,17 +30,18 @@ if __name__ == "__main__":
     from itertools import accumulate
     ## if LaueToolsNN is properly installed
     try:
-        from lauetoolsnn.utils_lauenn import get_multimaterial_detail, read_hdf5, new_MP_multimat_function, resource_path
+        from lauetoolsnn.utils_lauenn import get_multimaterial_detail, new_MP_multimat_function, resource_path, global_plots_MM
         from lauetoolsnn.lauetools import dict_LaueTools as dictLT
+        from lauetoolsnn.NNmodels import read_hdf5
     except:
         # else import from a path where LaueToolsNN files are
         import sys
         sys.path.append(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn")
-        from utils_lauenn import get_multimaterial_detail, read_hdf5, new_MP_multimat_function, resource_path         
+        from utils_lauenn import get_multimaterial_detail, new_MP_multimat_function, resource_path, global_plots_MM
+        from NNmodels import read_hdf5        
         sys.path.append(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\lauetools")
         import dict_LaueTools as dictLT
     
-    from keras.models import model_from_json
     import _pickle as cPickle
     from tqdm import tqdm
     
@@ -81,7 +82,7 @@ if __name__ == "__main__":
                     # =============================================================================
                     "experimental_directory": r"C:\Users\purushot\Desktop\Guillou_Laue\scan_0012",
                     "experimental_prefix": "ech1_map2D_3_",
-                    "grid_size_x" : 5,            ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
+                    "grid_size_x" : 1,            ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
                     "grid_size_y" : 5,  
                     "UB_tolerance": [0.6,
                                      0.6,
@@ -119,6 +120,7 @@ if __name__ == "__main__":
     strain_free_parameters = input_params["strain_free_parameters"]
     material_limit = input_params["material_ub_limit"]
     material_phase_always_present = input_params["material_phase_always_present"]
+    model_annote = "from_file"
     
     if len(material_) > 1:
         prefix_mat = material_[0]
@@ -127,7 +129,7 @@ if __name__ == "__main__":
                 continue
             prefix_mat = prefix_mat + "_" + imat
     else:
-        prefix_mat = material_
+        prefix_mat = material_[0]
     
     model_direc = os.getcwd()+"//"+prefix_mat+input_params["prefix"]
     
@@ -208,7 +210,6 @@ if __name__ == "__main__":
         config_setting.write(configfile)
     
     ## load model related files and generate the model
-    json_file = open(model_direc+"//model_"+prefix_mat+".json", 'r')
     classhkl = np.load(model_direc+"//MOD_grain_classhkl_angbin.npz")["arr_0"]
     angbins = np.load(model_direc+"//MOD_grain_classhkl_angbin.npz")["arr_1"]
     ind_mat_all = np.load(model_direc+"//MOD_grain_classhkl_angbin.npz",allow_pickle=True)["arr_5"]
@@ -217,19 +218,11 @@ if __name__ == "__main__":
         ind_mat.append(len(inni))
     ind_mat = [item for item in accumulate(ind_mat)]
     
+    # json_file = open(model_direc+"//model_"+prefix_mat+".json", 'r')
     load_weights = model_direc + "//model_"+prefix_mat+".h5"
     wb = read_hdf5(load_weights)
     temp_key = list(wb.keys())
-    
-    # # load json and create model
-    loaded_model_json = json_file.read()
-    json_file.close()
-    model = model_from_json(loaded_model_json)
-    print("Constructing model")
-    model.load_weights(load_weights)
-    print("Uploading weights to model")
-    print("All model files found and loaded")
-    
+
     ct = time.time()
     now = datetime.datetime.fromtimestamp(ct)
     c_time = now.strftime("%Y-%m-%d_%H-%M-%S")   
@@ -371,26 +364,25 @@ if __name__ == "__main__":
                 use_previous_UBmatrix_name,
                 material_phase_always_present,
                 crystal,
-                strain_free_parameters] for ii in range(count_global)]
+                strain_free_parameters,
+                model_annote] for ii in range(count_global)]
     
-    start_time = time.time()
-    # Launch on single file to verify
-    results = new_MP_multimat_function(valu12[0])
-    print('Took ',time.time()-start_time, "seconds")
+    # start_time = time.time()
+    # # Launch on single file to verify
+    # results = new_MP_multimat_function(valu12[0])
+    # print('Took ',time.time()-start_time, "seconds")
     
-    print("matching rate")
-    temp_0 = [results[6][i][0][0] for i in range(len(results[6]))]
-    print(temp_0)
-    
-    print("Orientation metrix")
-    temp_0 = [results[2][i][0][0].ravel() for i in range(len(results[6]))]
-    for itemm in temp_0:
-        print(",".join(str(x) for x in itemm))
-    
-    #indexing angles 787
+    # print("matching rate")
+    # temp_0 = [results[6][i][0][0] for i in range(len(results[6]))]
+    # print(temp_0)
     
+    # print("Orientation metrix")
+    # temp_0 = [results[2][i][0][0].ravel() for i in range(len(results[6]))]
+    # for itemm in temp_0:
+    #     print(",".join(str(x) for x in itemm))
+        
     #% Launch multiprocessing prediction     
-    if 0:
+    if 1:
         args = zip(valu12)
         with multiprocessing.Pool(ncpu) as pool:
             results = pool.starmap(new_MP_multimat_function, tqdm(args, total=len(valu12)))
@@ -420,7 +412,7 @@ if __name__ == "__main__":
                     fR_pix[intmat_mpdata][0][cnt_mpdata] = fR_pixel_mpdata[intmat_mpdata][0][cnt_mpdata]
                     best_match[intmat_mpdata][0][cnt_mpdata] = best_match_mpdata[intmat_mpdata][0][cnt_mpdata]
                 
-        #%% Save results
+        #% Save results
         save_directory_ = filenameDirec+"//results_"+prefix_mat+"_"+c_time
         if not os.path.exists(save_directory_):
             os.makedirs(save_directory_)
@@ -434,3 +426,10 @@ if __name__ == "__main__":
                               symmetry, crystal], output_file)
         print("data saved in ", save_directory_)
 
+        try:
+            global_plots_MM(lim_x, lim_y, rotation_matrix, strain_matrix, strain_matrixs, 
+                         col, colx, coly, match_rate, mat_global, spots_len, 
+                         iR_pix, fR_pix, save_directory_, material_,
+                         match_rate_threshold=5, bins=30)
+        except:
+            print("Error in the global plots module")
\ No newline at end of file
