@@ -17,7 +17,6 @@ def querymat():
     parser.add_argument("-n", "--name", required=True, help="User string for the material")
     args = parser.parse_args()
     config = vars(args)
-    print(config)
     
     mat_name = config["name"]
     
@@ -37,20 +36,21 @@ def querymat():
         extinction_json = json.load(f)
         
     ## Modify/ADD the dictionary values to add new entries
-    error_free = False
+    error_free_mat = False
+    error_free_ext = False
     try:
         material_queried = dict_Materials[mat_name]
-        error_free = True
+        error_free_mat = True
         try:
             _ = extinction_json[material_queried[2]]
-            error_free = True
+            error_free_ext = True
         except:
-            error_free = False
+            error_free_ext = False
             print("Extinction does not exist in the LaueNN library, please add it with lauenn_addmat command")
     except:
-        error_free = False
+        error_free_mat = False
         print("Material does not exist in the LaueNN library, please add it with lauenn_addmat command")
-    if error_free:
+    if error_free_mat:
         print("Material found in LaueNN library")
         print(material_queried)
 
@@ -102,3 +102,4 @@ def start():
         
     print("New material successfully added to the database with the following string")
     print(mat_name, lat, ext)
+    ## print dtype also
